/********************************************************
Workspace: IAR_ARM+SPL
MCU      : STM32f050f4/6
Project  : deafener
Version  :
Date     : SEP 2019
*********************************************************/

#include <stm32f0xx.h>
#include <stdbool.h>
#include <uchar.h>
#include <math.h>


#include "stm32f0xx_gpio.c"
#include "stm32f0xx_flash.c"
#include "stm32f0xx_dma.c"

#include "bitset.c"
#include "defines.c"
#include "periph_cfg.c"
#include "kb.c"
#include "led.c"
#include "beep.c"

#include "p_pwm.c"
#include "adc.c"

#include "interrupts.c"

void devcon(void);

void main()
{
 cfg_rcc();
 cfg_gpio();
 cfg_t1(PWM_TOP);
 cfg_dma_t1();
 cfg_adc();
 cfg_systic(1000);

 led(red, true);
 led(green, true);

 __enable_interrupt();
 NVIC_EnableIRQ(DMA1_Channel4_5_IRQn);
 NVIC_SetPriority(DMA1_Channel4_5_IRQn,0);
 NVIC_SetPriority(SysTick_IRQn,2);

 tim0 = 300;
 while(tim0)continue;

 //enable ram parity checking
 while(FLASH_OB_UBC(0xbf)==FLASH_BUSY)continue;

 //  PC_ON
 GPIOB->BSRR = 1<<1;

 while(1)
 {


  devcon();
 }
}

void devcon (void)
{

 static enum {wait, charge, hold, error} istate = wait;
 static bool istate_trg;
 static bool led_trg;

 #define CNLN 2
 #define ADC_FRQ adc_result[0]
 #define ADC_AMP adc_result[1]
 static u32 adc_result[CNLN];



 switch(istate)
 {
  case wait:
  {
   if(!istate_trg)
   {
    istate_trg = 1;

    SPWM_OFF;

    led(red, false);
    led(green, true);
    beep(1000, 300);
   }


   //   ,     
   static u32 adc_tmp[CNLN];
   static bool adc_ready;
   static int adc_cnt;

   if(do_conversion()==true)
   {
    adc_cnt++;
    for(int i=0;i<CNLN;i++) adc_tmp[i] += conversion_result[i];

    if(adc_cnt == 8)
    {
     adc_cnt = 0;
     adc_ready = 1;
     for(int i=0;i<CNLN;i++)
     {
      adc_result[i] = adc_tmp[i]>>3;
      adc_tmp[i] = 0;
     }
    }
   }
   //  

   if(btn[0]!=rls && adc_ready == true)
   {
    istate = charge;
    adc_ready = false;
   }
   if(istate != wait) istate_trg = 0;
  }break;

  case charge:
  {
   static int amp, tmp_amp;
   static int ibank;

   if(!istate_trg)
   {
    istate_trg = 1;

    int frq = WFRQ_MIN + (ADC_FRQ*(WFRQ_MAX - WFRQ_MIN)>>12);
    calculate_hf_sw(frq);

    amp = MAX_AMP*ADC_AMP>>12;
    tmp_amp = 0x0f;
    ibank = 0;

    led(green, false);
    beep(2000, 300);
   }

   //     
   //    
   if(!tim0)
   {
    tim0 = 100;

    if(tmp_amp <= amp/20) tmp_amp+= amp/20;
    else                  tmp_amp = amp;

    ibank ^= 1;
    set_wamp(tmp_amp,ibank);
    change_bank(ibank);

    if(!(TIM1->BDTR & TIM_BDTR_MOE)) SPWM_ON;
   }

   //,   
   if(!tim_led)
   {
    tim_led = 300;
    led_trg^=1;

    led(red, led_trg);
   }
   // 

   if(btn[0]==rls)         istate = wait;
   else if(tmp_amp >= amp) istate = hold;

   if(istate != charge) istate_trg = 0;
  }break;

  case hold:
  {
   if(!istate_trg)
   {
    istate_trg = 1;

    led(red, true);
   }

   //    
   if(!tim0)
   {
    tim0 = 1000;
    beep(4000, 100);
   }

   if(btn[0]==rls) istate = wait;
   if(istate != hold) istate_trg = 0;
  }break;

  case error:
  {
   if(!istate_trg)
   {
    istate_trg = 1;


   }

   if(!tim_led)
   {
    tim_led = 300;
    led_trg^=1;

    led(red, led_trg);
    led(green, !led_trg);
   }

   if(istate != error) istate_trg = 0;
  }break;
 }
}